/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package org.apache.myfaces.trinidadinternal.skin.pregen.context;

import org.apache.myfaces.trinidad.skin.Skin;
import org.apache.myfaces.trinidadinternal.skin.SkinStyleProvider;

/**
 * StyleProvider implementation used during skin pregeneration.
 * 
 * During skin pregeneration we periodically clear out the
 * StyleProvider in order to free up memory.  However, when
 * we do this, we don't want to force a re-parse of the skin
 * definitions.  We override getClearOnDirty() to avoid this.
 */
public class PregenStyleProvider extends SkinStyleProvider
{
  public PregenStyleProvider(Skin skin, String targetDirectoryPath)
    throws IllegalArgumentException
  {
    super(skin, targetDirectoryPath);
  }
  
  @Override
  protected ClearOnDirty getClearOnDirty()
  {
    // Only clear out cache entries - don't blow away the
    // StyleSheetDocument and associated data.
    return ClearOnDirty.ENTRIES;
  }
}
